#!/usr/local/bin/apl --script --

 ⍝⍝⍝⍝⍝⍝⍝⍝⍝⍝⍝⍝⍝⍝⍝⍝⍝⍝⍝⍝⍝⍝⍝⍝⍝⍝⍝⍝⍝⍝⍝⍝⍝⍝⍝⍝⍝⍝⍝⍝⍝⍝⍝⍝⍝⍝⍝⍝
⍝
⍝ cf 2014-04-21 16:37:26 (GMT-7)
⍝
 ⍝⍝⍝⍝⍝⍝⍝⍝⍝⍝⍝⍝⍝⍝⍝⍝⍝⍝⍝⍝⍝⍝⍝⍝⍝⍝⍝⍝⍝⍝⍝⍝⍝⍝⍝⍝⍝⍝⍝⍝⍝⍝⍝⍝⍝⍝⍝⍝

∇z←tn cf∆append data
 ⍝ Add a new component and return its component number.
 'insert into component ( data, vers, oid ) values ( ?, ?, ? )' SQL[4,tn] (cf⍙encode data) (cf∆version tn) (cf⍙next_cn tn)
 z←cf∆max_cn tn
∇

∇z←tn cf∆drop cn
 ⍝ Drop an existing component.
 'delete from component where oid = ?' SQL[4,tn] cn
 z←cn
∇

∇z←tn cf∆exists cn
 ⍝ Inquire whether a component exists.
 z←↑,'select 1 from component where oid = ?' SQL[3,tn] cn
∇

∇cf∆init
 ⍝ Ensure that the apl-sqlite library is loaded.
 →(0≠⎕nc 'SQL')/0
 cf⍙libpath ⎕fx 'SQL'
∇

∇z←cf∆max_cn tn
 ⍝ Return the component file's largest component number.
 z←↑'select current from max_cn where oid = 1' SQL[4,tn] ⍬
∇

∇z←tn cf∆read cn
 ⍝ Read an existing component.
 z←cf⍙decode ,⊃'select data from component where oid = ?' SQL[3,tn] cn
∇

∇z←where cf∆replace data;tn;cn
 ⍝ Replace an existing component.
 tn←1⊃where
 cn←2⊃where
 'replace into component ( oid, data ) values ( ?, ? )' SQL[4,tn] (cn) (cf⍙encode data)
 z←cn
∇

∇z←cf∆tie path
 ⍝ Get a reference to a named component file. Create the file if needed.
 z←'sqlite' SQL[1] path
 'create table if not exists component ( data text, vers int )' SQL[4,z] ⍬
 'create table if not exists version ( current int )' SQL[4,z] ⍬
 'create table if not exists max_cn ( current int )' SQL[4,z] ⍬
 ⍝ Set the initial maximum component number only at file creation.
 →(1=↑,'select 1 from max_cn where oid = 1' SQL[3,z] ⍬)/skip
 'insert into max_cn ( oid, current ) values ( 1, ? )' SQL[4,z] (0)
 skip:
 ⍝ Set the component file version only at file creation.
 →(1=↑,'select 1 from version where oid = 1' SQL[3,z] ⍬)/0
 'insert into version ( oid, current ) values ( 1, ? )' SQL[4,z] (1)
∇

∇z←cf∆untie tn
 ⍝ Close the referenced component file.
 SQL[2] tn
 z←⍬
∇

∇z←cf∆version tn
 ⍝ Return the component file's version number.
 z←↑'select current from version where oid = 1' SQL[3,tn] ⍬
∇

∇z←l (f cf∆with_transaction g) r;v
 ⍝ Execute f within a SQL transaction.
 ⍝ f is a dyadic database function; its left argument is either
 ⍝ the g in the case that the operator is called monadically
 ⍝ or the vector (g l) in the case that the operator is called
 ⍝ dyadically.
 ⍝ g must be a niladic function returning the database reference.
 ⍝ Returns a two-element vector: the first element is a boolean
 ⍝ scalar indicating success; the second element is the result of f.
 ⍝ Examples:
 ⍝ 2⊃cf∆append cf∆with_transaction {db} data
 ⍝ 2⊃cn cf∆replace cf∆with_transaction {db} data
 SQL[5] g ⍝ begin
 →(0≠⎕nc 'l')/dyadic
 v←'→rollback' ⎕ea 'g f r'
 →commit
 dyadic:
 v←'→rollback' ⎕ea '(g l) f r'
 commit:
 SQL[6] g ⍝ commit
 z←1 v
 →0
 rollback:
 SQL[7] g ⍝ rollback
 z←0 ⍬
∇

∇z←cf⍙decode data
 ⍝ This function is reserved to the implementation.
 ⍝ Decode from the database textual form to APL data.
 z←⍎data
∇

∇z←cf⍙encode data
 ⍝ This function is reserved to the implementation.
 ⍝ Encode APL data into a textual form for the database.
 z←5↓2⎕tf 'data'
∇

∇z←cf⍙next_cn tn
 ⍝ This function is reserved to the implementation.
 ⍝ Increment and return the component number.
 'replace into max_cn ( oid, current ) values ( 1, ? )' SQL[4,tn] 1+cf∆max_cn tn
 z←cf∆max_cn tn
∇

Describe←9⍴0
  Describe[0+⍳9]←'∘∘∘∘∘∘∘∘∘'
  ((⎕IO+0)⊃Describe)←⊂52⍴0
  ((⎕IO+0)⊃Describe)[⍳46]←'A simple component file system on top of apl-s'
  ((⎕IO+0)⊃Describe)[46+⍳6]←'qlite.'
  ((⎕IO+1)⊃Describe)←⊂60⍴0
  ((⎕IO+1)⊃Describe)[⍳46]←'Written for GNU APL by David B. Lamkins <david'
  ((⎕IO+1)⊃Describe)[46+⍳14]←'@lamkins.net>.'
  ((⎕IO+2)⊃Describe)←32⍴'http://www.gnu.org/software/apl/'
  ((⎕IO+3)⊃Describe)←19⍴'http://lamkins.net/'
  ((⎕IO+4)⊃Describe)←37⍴'https://github.com/lokedhs/apl-sqlite'
  ((⎕IO+5)⊃Describe)←23⍴'https://www.sqlite.org/'
  ((⎕IO+6)⊃Describe)←' '
  ((⎕IO+6)⊃Describe)←0⍴((⎕IO+6)⊃Describe)
  ((⎕IO+7)⊃Describe)←⊂54⍴0
  ((⎕IO+7)⊃Describe)[⍳46]←'This is a set of primitives intended to be use'
  ((⎕IO+7)⊃Describe)[46+⍳8]←'d either'
  ((⎕IO+8)⊃Describe)←⊂56⍴0
  ((⎕IO+8)⊃Describe)[⍳46]←'as-is or as building blocks for higher-level f'
  ((⎕IO+8)⊃Describe)[46+⍳10]←'acilities.'
  Describe←9 1⍴Describe

cf⍙libpath←21⍴'apl-sqlite/lib_sql.so'

⎕CT←1E¯13

⎕FC←6⍴(,⎕UCS 46 44 8902 48 95 175)

⎕IO←1

⎕L←0

⎕LX←' '
  ⎕LX←0⍴⎕LX

⎕NLT←1⍴'C'

⎕PP←10

⎕PR←1⍴' '

⎕PS←0

⎕PW←80

⎕R←0

⎕RL←1

⎕TZ←-7

